package Application;

import java.io.*;
import java.util.*;

/**
* Gestion du Compte de la societe, ecriture des lignes comptables
*
* @version 1.0, 2004/1/1
*/
public class Compte {

	/**
	* valeur de depart du compte
	*/
	protected Euro soldeInit;

	/**
	* valeur courante du compte
	*/
	protected Euro solde;

	/**
	* SDD contenant toutes les ecritures comptables passees
	*/
	protected Vector comp;

	/**
	* Constructeur du Compte
	*
	* @param soldeInit Solde initial du Compte
	* @param lignesComptablesInit Lignes comptables stockees dans un
	* fichier utilisees pour remettre le compte en etat
	*
	*/
	protected Compte(Euro soldeInit, Vector lignesComptablesInit) {

		this.soldeInit = (Euro)soldeInit.clone();

		/* Creation du nouveau Vector, et copie */
		this.comp = (Vector)lignesComptablesInit.clone(); 

		/* Mise a jour du solde. */
		this.majSolde();
	}

	/**
	* Vente d'un produit, enregistrement d'une ecriture comptable Credit
	*
	* @param nom Nom du produit vendu
	* @param qte Quantite vendue de ce produit
	* @param total Prix total en Euro (pour information)
	*/
	protected void vendreProduit(String nom, int qte, Euro total) { 

		/* Mise a jour du compte */
		this.solde.crediter(total);

		this.comp.add(new Vente(nom, qte, total));
	}

	/**
	* Annulation de la vente d'un produit,
	* enregistrement d'une ecriture comptable Debit
	*
	* @param nomProduit Nom du produit 
	* @param qte Quantite annulee pour ce produit
	* @param totalprix Prix total en euro
	*/
	protected void annulerVenteProduit(String nomProduit,
		int qte, Euro totalprix) {

		/* Mise a jour du compte */
		this.solde.debiter(totalprix);

		this.comp.add(new AnnulerVente(nomProduit, qte, totalprix));

	}

	/**
	* Depot d'une Somme d'argent
	*
	* @param nomPersonne Nom de la personne
	* @param Somme Somme d'argent deposee
	*/
	protected void Depot(String nomPersonne, Euro Somme) {
		this.solde.crediter(Somme);
		this.comp.add(new Depot(nomPersonne, Somme));
	}

	/**
	* Prelevement d'une Somme d'argent
	*
	* @param nomPersonne Nom de la personne
	* @param Somme Somme d'argent deposee
	*/
	protected void Prelevement(String nomPersonne, Euro Somme) {
		this.solde.debiter(Somme);
		this.comp.add(new Prelevement(nomPersonne, Somme));
	}

	/**
 	* Recherche de la presence d'un produit vendu avant annulation
	* on accepte si vente >= annulation + qte a annuler
	*
	* @param nomProduit Nom du produit
	* @param qte Quantite de produit
	* @return booleen indiquant si le produit a ete trouve en quantite
	* suffisante
	*/
	protected Vente rechercheVente(String nomProduit, int qte) {
		int i;
		LigneComptable t;
		String label, produit;
		int quantite;

		for(i = 0; i < comp.size(); i ++) {
			t = (LigneComptable)comp.get(i);
			label = t.getlabel();
			if(label.equals("vente_prod") == true) {
				produit = t.getproduit();
				quantite = t.getquantite();
				if(produit.equals(nomProduit) == true
				&& quantite >= qte)
					return((Vente)t);
			}
		}
		return(null);
	}

	/**
	* Mise a jour du solde du compte, Effectuee a partir du solde initial
	* et de toutes les ecritures comptables enregistrees
	*/
	protected void majSolde() {
		int i;
		LigneComptable t;
		String label;
		Euro total;

		/* On remet le solde initial a 0 */
		this.solde = (Euro)this.soldeInit.clone();

		/* Pour chaque entrees dans le Vecteur */
		for(i = 0; i < comp.size(); i ++) {
			t = (LigneComptable)comp.get(i);
			label = t.getlabel();
			total = t.gettotal();
			if(label.equals("vente_prod") == true) {
				this.solde.crediter(total);
			}
		}
	}

	/**
	* Effacement des Lignes comptables et mise a jour du solde 
	* avec le solde courant
	*/
	protected void reset() {
		this.comp.removeAllElements();
		this.solde = (Euro)this.soldeInit.clone();
	}

	/**
	* Obtention du solde du compte en Euro
	*/
	protected Euro solde() {
		return((Euro)this.solde.clone());
	}

	/**
	* Obtention de l'etat complet du Compte
	*
	* @return Chaine de caractere donnant l'etat du Compte
	*/
	public String toString() {
		return("Solde: " + solde + " ; Entrees : " + comp.size());
	}

	/**
	* Obtention du Vector des Lignes Comptables
	*
	* @return Un clone du Vector des lignes comptables
	*/
	public Vector getVector() {
		return((Vector)this.comp.clone());
	}
}

