package Application;

import java.io.*;
import java.util.*;

/**
* Classe de controle de l'application
*
* @author Patrick MARIE
* @version 1.1, 2004/1/10
*/
public class Controle {

	protected Compte cpt;
	private String nf_compte;

	private int verbose = 0;

	/**
	* Code d'erreur
	*/
	protected static final int OK = 0;
	protected static final int ERREUR_PERSONNE = -1;
	protected static final int MOT_DE_PASSE_INCORRECT = -2;

	/**
	* Constructeur Controle de l'application
	* @param initialcond nom du fichier stockant les informations sur le
	* montant du compte initial.
	* @param personnes nom du fichier stockant les informations sur les
	* Personnes
	* @param produits nom du fichier stockant les informations sur les
	* Produits
	* @param compte nom du fichier stockant l'etat courant du Compte
	*/
	public Controle(String initialcond, String personnes, 
		String produits, String compte) {

		double soldeInit;

		this.nf_compte = new String(compte); 
		soldeInit = this.getSoldeInit(initialcond);
		this.initPersonnes(personnes);
		this.initProduits(produits);
		this.initCompte(compte, new Euro(soldeInit));

	}

	/**
	* "Destructeur" du Controle: sauvegarde des donnees
	*/
	public void close() {
		this.sauverCompte();
	}	

	/**
	* chargement de SoldeInit via CondInit.txt (String)
	*
	* @param solde Fichier de conditions initiales
	* @return Le solde, en double.
	*/
	private double getSoldeInit(String solde) {

		FileReader sourceFile;
		BufferedReader istream;
		String s_prix;
		double d_prix = 0.0;

		try {
			sourceFile = new FileReader(solde);
			istream = new BufferedReader(sourceFile);
			s_prix = istream.readLine();
			if(s_prix != null) {
				d_prix = Double.valueOf(s_prix).doubleValue();
			} else {
				d_prix = 999.9;
			}
			istream.close();
			sourceFile.close();
		} catch (Throwable t) {
			System.out.println("erreur de fichier: " + solde);
			System.exit(0);
		}

		if (this.verbose != 0)
			System.out.println("Solde Initial: " + d_prix);

		return(d_prix);
	}

	/**
	* methode de chargement des informations concernant les Personnes
	*
	* @param personnes nom du fichier stockant les informations
	* sur les Personnes
	*/
	private void initPersonnes(String personnes) {
		
		FileReader sourceFile;
		BufferedReader istream;
		String s_nom, s_pass;

		try {
			sourceFile = new FileReader(personnes);
			istream = new BufferedReader(sourceFile);
			while((s_nom = istream.readLine()) != null) {
				if((s_pass = istream.readLine()) == null) {
					break;
				}

				if(this.verbose != 0)
					System.out.println("Ajout personne: (" + s_nom + ", " + s_pass + ")");

				SDDPersonne.ajouterPersonne(s_nom, s_pass);
			}

			istream.close();
			sourceFile.close();
		} catch(Throwable t) {
			System.out.println("erreur de fichier: " + personnes);
			System.exit(0);
		}
	}

	/**
	* methode de chargement des informations concernant les Produits
	*
	* @param produits nom du fichier stockant les informations sur les
	* Produits
	*/
	private void initProduits(String produits) {
		FileReader sourceFile;
		BufferedReader istream;
		String s_nom, s_prix;
		double d_prix;

		try {
			sourceFile = new FileReader(produits);
			istream = new BufferedReader(sourceFile);
			while((s_nom = istream.readLine()) != null) {
				if((s_prix = istream.readLine()) == null) {
					break;
				}

				d_prix = Double.valueOf(s_prix).doubleValue();

				if(this.verbose != 0)
					System.out.println("Ajout produit: (" + s_nom + ", " + d_prix);
				SDDProduit.ajouterProduit(s_nom,
					new Euro(d_prix));
			}
			istream.close();
			sourceFile.close();
		} catch (Throwable t) {
			System.out.println("erreur de fichier: " + produits);
			System.exit(0);
		}
	}

	/**
	* methode de chargement des informations concernant le Compte
	*
	* @param compte nom du fichier stockant l'etat courant du Compte
	* @param soldeInit valeur initiale du compte initialise a partir du
	* fichier
	*/
	private void initCompte(String compte, Euro soldeInit) {

		FileInputStream istream;
		ObjectInputStream q;
		Vector sddlc = new Vector();
		boolean loaded = false;
		Object to;
		int i = 0;

		try {
			istream = new FileInputStream(compte);
			q = new ObjectInputStream(istream);
			while((to = q.readObject()) != null) {
				if(this.verbose == 1) {
					System.out.println(to);
				}
				sddlc.add(to);
				i ++;
			}
			q.close();
			istream.close();
		} catch (Throwable t) {
			/*
			System.out.println(
			  "erreur de fichier: " + compte + " (non fatal.)");
			System.out.println(t);
			 */
		}

		if(this.verbose == 1) {
			System.out.println("Charge lc: " + sddlc.size() + " / " + i);
		}

		this.cpt = new Compte(soldeInit, sddlc);
	}

	/**
	* reset du Compte
	*/
	public void reset() {
		this.cpt.reset();
	}

	/**
	* sauvegarde de l'etat du compte
	*
	* @param compte Nom du fichier de compte
	*/
	public void sauverCompte(String compte) {

		FileOutputStream ostream;
		ObjectOutputStream p;
		Vector v = this.cpt.comp;
		int i = 0;

		try {
			ostream = new FileOutputStream(compte);
			p = new ObjectOutputStream(ostream);

			for ( ; i < v.size(); i ++) {
				p.writeObject(v.get(i));
			}
			p.flush();
			p.close();
			ostream.close();
		} catch(Throwable t) {
			System.out.println("erreur de fichier: " + compte);
			System.exit(0);
		}

		if(this.verbose == 1) {
			System.out.println("Sauve lc: " + v.size() + " / " + i);		}
	}

	/**
	* sauvegarde de l'etat du compte (surcharge, nom par defaut)
	*/
	public void sauverCompte() {
		this.sauverCompte(nf_compte);
	}

	/**
	* etatDuCompte fournit une chaine de caracteres representative 
	* de l'etat du compte
	*
	* @return le resultat de toString.
	*/
	public String etatDuCompte() {
		return (this.toString());
	}

	/**
	* Obtention de l'etat du Compte
	*
	* @return une chaine representant le compte
	*/
	public String toString() {
		return(this.cpt.toString());
	}

	/**
	* nomProduit fournit le nom du ieme Produit
	*
	* @param index rang du Produit dans la sdd de stockage
	*/
	public String nomProduit(int index) {
		return(cpt.toString());
	}

	/**
	* nomProduit fournit le nom du Produit donne
	*
	* @param produit Le Produit lui meme
	*/
	public String nomProduit(Produit produit) {
		return(produit.nom());
	}

	/**
	* nbProduits fournit le nombre de produits differents dans la caisse
	*
	* @return le nombre de produits
	*/
	public int nbProduits() {
		return(SDDProduit.size());
	}

	/**
	* Prelevement d'une somme d'argent
	*
	* @param nomPersonne nom de la Personne
	* @param motPasse le mot de passe
	* @param Somme Somme en Euro prelevee
	*
	* @return Le status
	*/
	public int nouveauPrelevement(String nomPersonne, String motPasse,
		Euro Somme) {

		Personne pers;

		/* On recupere l'objet Personne, si il existe */
		pers = SDDPersonne.donnePersonne(nomPersonne);
		if(pers == null) { return(ERREUR_PERSONNE); }

		if(pers.motDePasseOK(motPasse) == false) {
			return(MOT_DE_PASSE_INCORRECT);
		}

		/* On ajoute une ligen au compte sur ce prelevement */
		this.cpt.Prelevement(nomPersonne, Somme);

		return(OK);
	}
	
	/**
	* Depot d'une somme d'argent d'un client
	*
	* @param nomPersonne nom de la Personne
	* @param motPasse le mot de passe
	* @param Somme Somme en Euro depose
	*
	* @return Le status
	*/
	public int nouveauDepot(String nomPersonne, String motPasse,
		Euro Somme) {

		Personne pers;

		/* On recupere l'objet Personne, si existe. */
		pers = SDDPersonne.donnePersonne(nomPersonne);
		if(pers == null) { return(ERREUR_PERSONNE); }

		if(pers.motDePasseOK(motPasse) == false) {
			return(MOT_DE_PASSE_INCORRECT);
		}

		/* On ajoute une ligne au compte sur ce depot */
		this.cpt.Depot(nomPersonne, Somme);

		return(OK);
	}

	/**
	* nouvelleVente methode de prise en compte de la vente d'une quantite
	* d'un Produit
	* @param nomProduit nom du Produit qui a ete vendu
	* @param qte quantite de Produit vendu
	*/
	public void nouvelleVente(String nomProduit, int qte) {

		int qtevendue = qte;
		Euro totalprix;
		Personne pers;
		Produit prod;
		double argent;
		boolean price_ok = false;

		if (qte <= 0) { return ; }

		/* On recupere le Produit correspondant */
		prod = SDDProduit.rechProduit(nomProduit);
		if(prod == null) { return ; }

		totalprix = new Euro(qtevendue * prod.prixU().valeur());

		if(this.verbose != 0) {
			System.out.println("Vente de " + qtevendue + " " + nomProduit + " pour " + totalprix);
		}

		/* On ajoute au compte une notification sur la vente */
		this.cpt.vendreProduit(nomProduit, qtevendue, totalprix);
	}

	/**
	* annulerVenteProduit, annulation de la vente de produit
	*
	* @param nomProduit Le nom du produit a annuler
	* @param qte La quantite du produit a annuler
	*/
	public void annulerVenteProduit(String nomProduit, int qte) {
		Produit prod;
		Vente vente;
		Euro totalprix;

		/* On recupere la ligne comptable de la vente */
		if((vente = this.cpt.rechercheVente(nomProduit,qte)) == null) {
			return ;
		}

		/* On recupere le Produit, on en a besoin pour le calcul */
		if((prod = SDDProduit.rechProduit(nomProduit)) == null) {
			return ;
		}

		totalprix = new Euro(qte * prod.prixU().valeur());

		if(this.verbose != 0) {
			System.out.println("Annulation de Vente de " + qte + " "+ nomProduit);
		}

		/* Ligne comptable */
		this.cpt.annulerVenteProduit(nomProduit, qte, totalprix);
	}

	/**
	* Choisir le niveau verbeux des messages d'erreur de l'application
	*
	* @param n Niveau verbeux
	*/
	public void setVerbose(int n){
		this.verbose = n;
	}	

	/**
	* Renvoit le Vector des Produits
	*
	* @return Un clone du Vector de Produit
	*/
	public Vector getProduitVector() {
		return(SDDProduit.getVector());
	}

	/**
	* Renvoit le Vector des Personnes
	*
	* @return Un clone du Vector de Personne
	*/
	public Vector getPersonneVector() {
		return(SDDPersonne.getVector());
	}

	/**
	* Renvoit le Vector des LignesComptable 
	*
	* @return Un clone du Vector de LigneComptables
	*/
	public Vector getLignesComptables() {
		return(this.cpt.getVector());
	}
}
