package Application;

/**
* Operations autorisees sur des sommes en Euro
*
* @version 1.0, 2003/11/18
* @see Comparable
* @see java.lang.Cloneable
*/
public class Euro implements java.io.Serializable, java.lang.Cloneable, Comparable
{
	/**
	* La somme est stockee sous forme d'euro centimes
	*/
	protected int eurocentimes;

	/**
	* Constructeur de Euro; initialise la valeur de Euro.
	*
	* @param valInit La valeur initiale.
	*/
	public Euro (double valInit) {
		this.eurocentimes = (int)(valInit * 100.);
	}

	/**
	* Augmenter la somme
	*
	* @param montant L'objet Euro contenant la valeur a ajouter
	*/
	public void crediter(Euro montant) {
		this.eurocentimes += montant.valeur() * 100;
	}

	/**
	* Debiter la somme
	*
	* @param montant L'objet Euro contenant la somme a retirer
	*/
	public void debiter(Euro montant) {
		this.eurocentimes -= montant.valeur() * 100;
	}

	/**
	* Fournit la somme convertie en doubles 
	*
	* @return Valeur de la somme convertie en double
	*/
	public double valeur() {
		return((double)this.eurocentimes / 100);
	}

	/**
	* Representation en chaine de caracteres de la somme 
	*
	* @return Chaine de caracteres representative de la somme
	*/
	public String toString() {
		return(this.valeur() + " E");
	}

	/**
	* Comparaison entre un objet et la valeur actuelle
	*
	* @param obj L'objet a comparer avec la classe
	* @return False si la valeur d'obj n'est pas inferieur
	* True si elle l'est.
	*/
	public boolean inf(Object obj) {

		/* Si l'objet n'est pas de type Euro, alors on compare pas */
		if (obj instanceof Euro == false)
			return(false);

		/* Si l'objet est inferieur, alors on renvoit true */
		if (((Euro)obj).valeur() < this.valeur())
			return(true);

		/* Sinon on renvoit false */
		return(false);
	}

	/**
	* Clonage de l'instance courante
	*
	* @return Clone de l'instance courante
	*/
	public Object clone() {
		Euro n = new Euro(this.valeur());
		return(n);	
	}
}
