package Application;

/**
* @version 1.0, 2004/1/1
*/
abstract public class LigneComptable implements java.io.Serializable {

	/**
	* Nom du Produit ou de la Personne
	*/
	protected String nom = new String("NoName");

	/**
	* Total de la transaction
	*/
	protected Euro total;

	/**
	* Quantite de Produit vendu ou annule
	*/
	protected int quantite;

	/**
	* Prix unitaire du Produit concerne
	*/
	protected Euro prixUnitaire;

	/**
	* label indiquant la nature de la transaction
	*/
	protected String label;

	/**
	* Construit une LigneComptable a partir d'un label
	* et d'un nom de personne
	*
	* @param label Label indiquant la nature de la transaction
	* @param nom Nom de la Personne
	*/
	protected LigneComptable(String label, String nom) {
		this.label = new String(label);
		this.nom = new String(nom);
	}

	/**
	* Construit une LigneComptable a partir d'un label
	* et d'un nom de produits
	* Methode pour label: "depot", "prelevement"
	*
	* @param label Label indiquant la nature de la transaction
	* @param nom Nom du Produit
	* @param total Total en Euro
	*/
	protected LigneComptable(String label, String nom, Euro total) {
		this.label = new String(label);
		this.nom = new String(nom);
		this.total = (Euro)total.clone();
	}

	/**
	* Construit une LigneComptable a partir d'un label
	* et d'un nom de produit
	*
	* @param label Label indiquant la nature de la transaction
	* @param nom Nom du Produit
	* @param qte Quantite de Produit
	* @param total Total en Euro
	*/
	protected LigneComptable(String label, String nom, int qte, 
		Euro total) {

		this.label = new String(label);
		this.nom = new String(nom);
		this.quantite = qte;
		this.total = (Euro)total.clone();
	}	

	/**
	* Construit une LigneComptable a partir d'un label
	* et d'un nom de produits
	* Methode pour label: "vente_prod", "annuler_vente_prod"
	*
	* @param label Label indiquant la nature de la transaction
	* @param nom Nom du Produit
	* @param client Nom du client
	* @param qte Quantite de Produit
	* @param total Total en Euro
	*/
	protected LigneComptable(String label, String nom, String client,
		int qte, Euro total) {

		this.label = new String(label);
		this.nom = new String(nom);
		this.quantite = qte;
		this.total = (Euro)total.clone();
	}

	/**
	* Retourne la somme totale de la transaction en Euro
	*
	* @return Valeur du total en Euro
	*/
	protected Euro total() {
		return((Euro)total.clone());
	}

	/**
	* Retourne une chaine de caracteres representative de la LigneComptable
	* courante
	*
	* @return Chaine de caractere representative de la LigneComptable
	*/
	public String toString() {
		return (label + ": " + nom + " (" + total + ") ");
	}

	/**
	* Retourne le label de la ligne comptable
	*
	* @return Le label
	*/
	public String getlabel() {
		return(new String(label));
	}

	/**
	* Retourne le total en Euro de cette ligne comptable
	*
	* @return Le total en Euro
	*/
	public Euro gettotal() {
		return((Euro)this.total.clone());
	}

	/**
	* Retourne le produit de cette ligne comptable
	*
	* @return Le produit, ou "NoProduit"
	*/
	public String getproduit() {
		return(new String(nom));
	}

	/**
	* Retourne la quantite affectee a cette ligne comptable pour le
	* produit
	*
	* @return La quantite
	*/
	public int getquantite() {
		return(this.quantite);
	}
}
