package Application;

import java.util.*;

/**
* Une Personne est caracterisee par son nom et son mot de passe
* Methodes Statiques.
*
* @version 1.0, 2003/11/27
*/
public class SDDPersonne {
	/**
	* Code d'erreur
	*/
	protected static final int ERREUR_PERSONNE = -1;
	protected static final int MOT_DE_PASSE_INCORRECT = -2;

	/**
	* Liste des Personens autorisees a effectuer un Prelevement
	*/
	protected static Vector personnes;

	public SDDPersonne() {

	}

	/**
	* Methode statique: Permet d'ajouter une Personne dans la liste
	*
	* @param nom Chaine de caracteres donnant le nom de la Personne
	* @param motDePasse Mot de passe permettant de verifier les droits
	*/
	protected static void ajouterPersonne(String nom, String motDePasse) { 
		if(!(personnes instanceof Vector))
			personnes = new Vector();

		personnes.add(new Personne(nom, motDePasse));
	}

	/**
	* Methode statique: Permet de connaitre le nombre de Personnes
	* dans la liste
	*
	* @return nombre de Personnes presents dans la liste
	*/
	protected static int size() {
		return(personnes.size());
	}

	/**
	* Methode statique: Retourne la ieme Personne de la liste
	*
	* @param index L'index de la Personne voulue
	* @return Reference de la ieme Personne
	*/
	protected static Personne personneAt(int index) {
		return((Personne)(personnes.elementAt(index)));
	}

	/**
	* Methode statique: Retourne l'index de la Personne connue par son nom
	*
	* @param n Le nom de la personne
	* @return Index correspondant a la Personne
	*/
	protected static int rechPersonne(String n) {
		int i;

		for(i = 0; i < personnes.size(); i ++) {
			if (personneAt(i).nom().equals(n) == true)
				return(i);
		}
		return(ERREUR_PERSONNE);
	}

	/**
	* Methode statique: Retourne la Personne de nom donne
	*
	* @param n Le nom de la Personne
	* @return L'objet Personne en question
	*/
	protected static Personne donnePersonne(String n) {
		int u;
		if ((u = rechPersonne(n)) != -1)
			return(personneAt(u));
		else
			return(null);
	}

	/**
	* Methode statique: Recherche si le mot de passe correspond bien
	* a la Personne donnee par son nom
	*
	* @param n Nom de la personne cherchee
	* @param motDePasse Mot de passe associe a la Personne
	* @return Booleen indiquant la validite du mot de passe
	*/
	protected static boolean rechPersonne(String n, String motDePasse) {
		int i;
		Personne tmp;

		i = rechPersonne(n);
		if(i == -1)
			return(false);
		else {
			tmp = personneAt(i);
			if(tmp.motDePasseOK(motDePasse) == true) {
				return(true);
			}
		}
		return (false);
	}

	/**
	* Methode statique: fournit le nom de la ieme Personne
	*
	* @param index rang de la Personne dans la sdd de stockage
	*/
	public static String nomPersonne(int index) {
		return(personneAt(index).nom());
	}

	/**
	* Methode statique: Retourne une chaine de caracteres representant
	* le contenu de la liste
	*
	* @return chaine de caracteres representative du contenu de la liste
	*/
	protected static String image() {
		StringBuffer chaine = new StringBuffer();
		int i;

		for(i = 0; i < personnes.size(); i ++) {
			if(i != 0)
				chaine.append(" - ");
			chaine.append(personneAt(i).nom());
		}
		return(chaine.toString());
	}

	/**
	* Methode statique : Renvoit le Vector des Personnes
	* (Pour affichage)
	*
	* @return un clone du Vector de Personne
	*/
	protected static Vector getVector() {
		return((Vector)personnes.clone());
	}
}
